function AppListAssistant(item)
{
	this.item = item;
	this.listModel = {items:[]};
}

AppListAssistant.prototype.setup = function()
{
	// setup command menu
	this.cmdMenuModel =
	{
		label: $L('Menu'), 
		items: [{label: $L('Back'), icon:'back', command: 'back'}]
	};
	this.controller.setupWidget(Mojo.Menu.commandMenu, undefined, this.cmdMenuModel);
	
	// setup list title
	this.controller.get('listTitle').innerHTML = this.item.name;
	
	// build list from global array
	for (var a = 0; a < apps.length; a++) 
	{
		var pushIt = false;
		
		// all
		if (this.item.list == 'all') pushIt = true;
		
		// updates
		if (this.item.list == 'updates' && apps[a].Update) pushIt = true;
		
		// installed
		if (this.item.list == 'installed' && apps[a].Installed) pushIt = true;
		
		// category
		if (this.item.list == 'category' && this.item.category == apps[a].Section) pushIt = true;
		
		// push it to the list if we should
		if (pushIt) 
		{
			// check for icon in SourceObj
			var tmpApp = apps[a];
			if (tmpApp.SourceObj != undefined && tmpApp.SourceObj.Icon)
			{
				tmpApp.ListIconClass = 'img';
				tmpApp.ListIconImg = '<img src="' + tmpApp.SourceObj.Icon + '" />';
			}
			
			// push
			this.listModel.items.push(tmpApp);
		}
	}
	
	// setup list attributes
	if (this.item.list == 'all')
	{
		// all list has alpha dividers
		this.listAttributes = {
			itemTemplate: "app-list/rowTemplate",
			swipeToDelete: false,
			reorderable: false,
			dividerTemplate: 'app-list/rowAlphaDivider',
			dividerFunction: this.getAlphaDivider
		};
	}
	else
	{
		// default list has no dividers
		this.listAttributes = {
			itemTemplate: "app-list/rowTemplate",
			swipeToDelete: false,
			reorderable: false
		};
	}
	
	// change scene if this is a single category
	if (this.item.list == 'category')
	{
		// update submenu styles
		this.controller.get('appListHeader').className = 'palm-header left';
		this.controller.get('categorySource').style.display = 'inline';
		this.controller.get('categoryTitle').innerHTML = this.item.category;
		
		// listen for tap to open menu
		Mojo.Event.listen(this.controller.get('categorySource'), Mojo.Event.tap, this.menuTapHandler.bindAsEventListener(this));
	}
	
	// setup list widget
	this.controller.setupWidget('appList', this.listAttributes, this.listModel);
	
	// listen for tap
	Mojo.Event.listen(this.controller.get('appList'), Mojo.Event.listTap, this.listTapHandler.bindAsEventListener(this));
	
}

AppListAssistant.prototype.getAlphaDivider = function(item)
{
	var firstChar = item.Description.substr(0, 1);
	if (parseInt(firstChar) == firstChar)
	{
		return '#';
	}
	else
	{
		return firstChar.toUpperCase();
	} 
}

AppListAssistant.prototype.listTapHandler = function(event)
{
	// push app view scene with this items info
	this.controller.stageController.pushScene('app-view', event.item);
}

AppListAssistant.prototype.menuTapHandler = function(event)
{
	// build category list model
	var categoryMenu = [];
	for (var c = 0; c < cats.length; c++) 
	{
		categoryMenu.push(
		{
			label: cats[c].name.substr(0, 1).toUpperCase() + cats[c].name.substr(1),
			command: cats[c].name
		});
	}
	
	// open category selector
	this.controller.popupSubmenu(
	{
		onChoose: function(value)
		{
			if (value === null ||
				value == "" ||
				value == undefined ||
				value == this.item.category) 
			{
				return;
			}
			else
			{
				this.controller.stageController.swapScene('app-list', {list: 'category', category: value.toLowerCase(), name: "WebOS Applications"});
				return;
			}
			
		},
		toggleCmd: this.item.category,
		placeNear: event.target,
		items: categoryMenu
	});
}

AppListAssistant.prototype.handleCommand = function(event)
{
	if(event.type == Mojo.Event.command)
	{
		if (event.command == 'back')
		{
			this.controller.stageController.popScene();
		}
	}
}

AppListAssistant.prototype.activate = function(event) {}

AppListAssistant.prototype.deactivate = function(event) {}

AppListAssistant.prototype.cleanup = function(event) {}
